# -*- coding: utf-8 -*-
"""
Define IceShelf class and initialise grid

S Buzzard created, lat updated 21/06/2021

@author: sglsb6
"""

#Set up class
class IceShelf:
    # Initializer / Instance Attributes
    rho_ice= 913
    rho_water=1000
    L_ice = 334000
    pore_closure= 830
    k_air=0.022; 
    cp_air=1004;
    k_water=0.5818;
    cp_water=4217;
    def __init__(self, lat, lon, S, M, M2, rho, T, Sfrac='init',
                 Lfrac='init', meltflag='init', saturation='init', 
                 lake_depth=0, T_lake=273.15, lid_depth=0, T_lid='init', 
                 melt='False', exposed_water='False', lake='False', v_lid='False', lid='False', 
                 water_level=0, ice_lens='False'):       
        #Variables
        self.lat = lat #Latitude
        self.lon = lon #Longitude
        self.S = S #Firn column height
        self.rho = rho #Initial density only, after this use only Sfrac
        self.T = T #Firn temperature
        if Sfrac=='init': #Solid fraction (0 to 1) 
            Sfrac= np.ones(M)*rho/913 
        self.Sfrac = Sfrac
        if Lfrac=='init': #Liquid fraction (0 to 1)  
            Lfrac= np.zeros(M) 
        self.Lfrac = Lfrac
        if meltflag=='init': #Melt has reached here (0 or 1)  
            meltflag= np.zeros(M)
        self.meltflag = meltflag
        if saturation=='init': #all pore space saturated with meltwater (0 or 1)  
            saturation= np.zeros(M)
        self.saturation = saturation
        self.lake_depth = lake_depth
        self.T_lake= T_lake #lake temperature (well mixed)
        self.Lid_depth= lid_depth
        if T_lid=='init': #Liquid fraction (0 to 1)  
            T_lid= np.ones(M2)*273.15 
        self.T_lid = T_lid 
        self.water_level=water_level
        
        #States model can be in 
        self.melt= melt #melt has occurred but not saturated
        self.exposed_water= exposed_water #Firn is saturated to surface
        self.lake= lake #Lake present
        self.v_lid = v_lid #Virtual frozen lid on lake
        self.lid = lid #True frozen lid on lake
        self.ice_lens= ice_lens #Frozen water has caused ice lens formation(no percolation below this)
        
        
def rho_init(z, rho_sfc, z_t):
    rho= 917-(917-rho_sfc)*np.exp(-(1.9/z_t)*z)
    return rho