# -*- coding: utf-8 -*-
"""
Define IceShelf class and initialise grid

S Buzzard created, last updated 06/07/2021

"""


# Set up class
class IceShelf:
    # Initializer / Instance Attributes
    rho_ice = 913
    rho_water = 1000
    L_ice = 334000
    pore_closure = 830
    k_air = 0.022
    cp_air = 1004
    k_water = 0.5818
    cp_water = 4217

    def __init__(self, x, y, s, m, m2, rho, T, Sfrac='init',
                 Lfrac='init', meltflag='init', saturation='init',
                 lake_depth=0, T_lake=273.15, lid_depth=0, T_lid='init',
                 melt='False', exposed_water='False', lake='False', v_lid='False', lid='False',
                 water_level=0, ice_lens='False'):
        # Variables
        self.x = x  # x coordinate of cell on grid
        self.y = y  # y coordinate of cell on grid
        self.s = s  # Firn column height
        self.m = m  # Vertical grid cell size for firn
        self.m2 = m2  # Vertical grid cell size for lake/ ice lid
        self.rho = rho  # Initial density only, after this use only Sfrac
        self.T = T  # Firn temperature
        if Sfrac == 'init':  # Solid fraction (0 to 1)
            Sfrac = np.ones(int((s/m)+1)) * rho / 913
        self.Sfrac = Sfrac
        if Lfrac == 'init':  # Liquid fraction (0 to 1)
            Lfrac = np.zeros(int((s/m)+1))
        self.Lfrac = Lfrac
        if meltflag == 'init':  # Melt has reached here (0 or 1)
            meltflag = np.zeros(int((s/m)+1))
        self.meltflag = meltflag
        if saturation == 'init':  # all pore space saturated with meltwater (0 or 1)
            saturation = np.zeros(int((s/m)+1))
        self.saturation = saturation
        self.lake_depth = lake_depth
        self.T_lake = T_lake  # lake temperature (well mixed)
        self.Lid_depth = lid_depth
        if T_lid == 'init':  # Liquid fraction (0 to 1)
            T_lid = np.ones(int((lid_depth/m)+1)) * 273.15
        self.T_lid = T_lid
        self.water_level = water_level

        # States model can be in
        self.melt = melt  # melt has occurred but not saturated
        self.exposed_water = exposed_water  # Firn is saturated to surface
        self.lake = lake  # Lake present
        self.v_lid = v_lid  # Virtual frozen lid on lake
        self.lid = lid  # True frozen lid on lake
        self.ice_lens = ice_lens  # Frozen water has caused ice lens formation(no percolation below this)


def rho_init(z, rho_sfc, z_t):
    rho = 917 - (917 - rho_sfc) * np.exp(-(1.9 / z_t) * z)
    return rho

class Grid(object):
    def __init__(self, col_amount, row_amount):

        self.cell_list = []  # empty list to hold all

        #Cell numbers run from bottom left from left to right, then down to up

        for col in range(col_amount):
            for row in range(row_amount):
                # create each cell as instance of Cell,
                # with the concatenated col and row numbers
                # as the name, then add the cell.name to cell_list
                cell = IceShelf(col, row, s, m, m2, rho, T, Sfrac='init',
                 Lfrac='init', meltflag='init', saturation='init',
                 lake_depth=0, T_lake=273.15, lid_depth=0, T_lid='init',
                 melt='False', exposed_water='False', lake='False', v_lid='False', lid='False',
                 water_level=0, ice_lens='False')
                self.cell_list.append(cell)



#Example small grid initialisation
# # make an instance of Grid with size 10x10
# s=20
# m=0.2
# z=np.linspace(0,s,int((s/m)+1))
# m2=0.2
# z_t=37
# rho_sfc=500
# rho=rho_init(z, rho_sfc, z_t)
# T=z+250
# grid = Grid(10, 10)
# print(len(grid.cell_list))
#
# print(grid.cell_list[2].x)
# print(grid.cell_list[2].y)




