# All functions related to the lateral transport of meltwater in the MONARCHS model


def add_random_water(cells, max_grid_row, max_grid_col):
    "For testing- adds a random volume of water to surface of each cell"
    
    rand_water=np.random.rand(max_grid_row,max_grid_col)/1000
    #Done in 2 steps to allow percolation to be added later
    cells= [[IceShelf(cells[i][j].height,(cells[i][j].water+rand_water[i][j]),rand_water[i][j]) for j in range(max_grid_col)] for i in range(max_grid_row)]
    cells= [[IceShelf((cells[i][j].height+rand_water[i][j]),cells[i][j].water,0) for j in range(max_grid_col)] for i in range(max_grid_row)]
    return cells


def move_water(cells, max_grid_row, max_grid_col, grid_size, timestep):
    "Identifies the surrounding cells with lowest height and moves water to it/them"
    for i in range(0, max_grid_row):
        for j in range(0, max_grid_col):
            if cells[i][j].water > 0: #Calc height difference between central cell and neighbours
                # neighbour 1,2,3,4 N,E,S,W
                if i > 0:
                    neighbour1 = cells[i][j].height - cells[i - 1][j].height
                else:
                    neighbour1 = -999
                if i < max_grid_row - 1:
                    neighbour3 = cells[i][j].height - cells[i + 1][j].height
                else:
                    neighbour3 = -999
                if j > 0:
                    neighbour4 = cells[i][j].height - cells[i][j - 1].height
                else:
                    neighbour4 = -999
                if j < max_grid_col - 1:
                    neighbour2 = cells[i][j].height - cells[i][j + 1].height
                else:
                    neighbour2 = -999
                m = max([neighbour1, neighbour2, neighbour3, neighbour4]) #Find neightbour with biggest height difference
                max_list = [a for a, b in enumerate([neighbour1, neighbour2, neighbour3, neighbour4]) if b == m]
                split = len(max_list)#If more than one cell is equally lower than central the water is split between them
                if m > 0: #Water moves if one of surrounding cells is lower than central cell
                    water_move = water_fraction(cells[i][j], m, grid_size, timestep) #How much can move within the timestep
                    if water_move * split > m:
                        water_move = (m / (split + 1))*split  # Not all water can move (i.e. it makes the destination surface equal in height to the source) so a fraction stays behind
                    if water_move * split > cells[i][j].water:
                        water_move = cells[i][j].water #More water could move than is available so all moves
                    for k in max_list: #Move the water to new location
                        if k == 0:
                            cells[i - 1][j].new_water = cells[i - 1][j].new_water + water_move / split
                        if k == 1:
                            cells[i][j + 1].new_water = cells[i][j + 1].new_water + water_move / split
                        if k == 2:
                            cells[i + 1][j].new_water = cells[i + 1][j].new_water + water_move / split
                        if k == 3:
                            cells[i][j - 1].new_water = cells[i][j - 1].new_water + water_move / split
                    cells[i][j].water = cells[i][j].water - water_move #Update water content and height of central cell
                    cells[i][j].height = cells[i][j].height - water_move

    # Have to do this adjustment last because of the speed limiting!
    # Would add percolation in here instead of just adding all new water (which is why this is currently done in 2 steps!)
    cells = [[IceShelf(cells[i][j].height, (cells[i][j].water + cells[i][j].new_water), cells[i][j].new_water) for j in
              range(max_grid_col)] for i in range(max_grid_row)]
    cells = [[IceShelf((cells[i][j].height + cells[i][j].new_water), cells[i][j].water, 0) for j in range(max_grid_col)]
             for i in range(max_grid_row)]

    return cells

def water_fraction(cell, m, grid_size, timestep):
    Big_pi=-2.53*10**-10 #hydraulic permeability (m^2)
    eta=1.787*10**-3#viscosity(Pa/s)
    u=Big_pi/eta*m*cell.rho*-9.8/grid_size #flow speed (m/s)
    water_frac=u*timestep/grid_size
    if u>1:
        u=1 #All water can move in one timestep
    water_move=cell.water*water_frac
    return water_move