#This example code runs the basic model functionality (vertical heat transfer, lateral water movement)
#on a small enough grid for testing on a standard laptop. Larger grids and full functionality require HPC and parallelisation

import Initialise_grid
import Lateral_Functions
import Vertical_Functions
#Under final development stages:
#import Pre_Lake_Functions.py
#import Lake_Functions.py
#import Lid_Functions.py

import numpy as np
from scipy.optimize import fsolve


#Initialise the grid


#Example small grid initialisation
#Make an instance of Grid with size 10x10
s = 20
m = 0.2
z = np.linspace(0,s,int((s/m)+1))
m2 = 0.2
z_t = 37
rho_sfc = 500
rho = Initialise_grid.rho_init(z, rho_sfc, z_t)
T = z+250
grid = Initialise_grid.Grid(10, 10, s, m, m2, rho, T)

#Test extract variable
#print(grid.cell_list[2].T)

#Give sample grid random heights:
for i in range(100):
    grid.cell_list[i].s = s*np.random.rand(1)

#Start out with some initial randomly distirbuted surface water
grid=Lateral_Functions.add_random_water(grid, max_grid_row, max_grid_col)

#Assume postive surface energy balance for demonstration with example weather conditions:
wind = 2
T_air = 267+3*np.random.rand()
s_hum = 2
p_air = 980
LW_in = 200
SW_in = 200
#Run heat equation solver and lateral water movement (once per hour)
for hours in range (2):
    for t_steps in range(360): #Run for one hour with 10 sec timestep
        dt = 10
        dz = 0.1
        for i in range(100):
            x = grid.cell_list[i].T  # x is initial guess, currently using value from prev timestep
            root = fsolve(Vertical_Functions.heateqn, x, args=(grid.cell_list[i], dt, dz, LW_in, SW_in, T_air, p_air, s_hum, wind))
            if root[0] > 273.15:
                root[0] = 273.15  # in full code then goes back and re-calculates with T_sfc set to 273.15- this is fast as it's Dirichlet
    Lateral_Functions.cells=move_water(grid, 10,10)
    Lateral_Functions.plot_water(grid,10,10)




#Workflow of full model:
#For each x timesteps
#     #For each grid cell:
#     if not lake:
#     #SEB and heat eqn
#     #Percolation
#     if pre-lake:
#     #SEB and heat eqn
#     if lake
#     Lake SEB
#     #Lake/lid development
#     if lid
#     Lid SEB
#     #Lake/lid development
# #Then run lateral flow once

